@@ -100,6 +100,9 @@ static void
 sp_application_startup (GApplication *application)
 {
   g_autoptr(GtkCssProvider) provider = NULL;
+#ifdef DEVELOPMENT_BUILD
+  g_autoptr(GtkCssProvider) adwaita = NULL;
+#endif
 
   g_assert (SP_IS_APPLICATION (application));
 
@@ -111,6 +114,14 @@ sp_application_startup (GApplication *application)
                                              GTK_STYLE_PROVIDER (provider),
                                              GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 
+#ifdef DEVELOPMENT_BUILD
+  adwaita = gtk_css_provider_new ();
+  gtk_css_provider_load_from_resource (adwaita, "/org/gnome/sysprof/theme/Adwaita-shared.css");
+  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
+                                             GTK_STYLE_PROVIDER (adwaita),
+                                             GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+#endif
+
   for (guint i = 0; default_accels[i].action_name; i++)
     gtk_application_set_accels_for_action (GTK_APPLICATION (application),
                                            default_accels[i].action_name,
@@ -172,12 +183,12 @@ sysprof_about (GSimpleAction *action,
                          "artists", sysprof_artists,
                          "comments", _("A system profiler"),
                          "copyright", "Copyright © 2004-2009 Søren Sandmann Pedersen\n"
-                                      "Copyright © 2016 Christian Hergert",
+                                      "Copyright © 2016-2018 Christian Hergert",
                          "transient-for", best_toplevel,
                          "modal", TRUE,
                          "translator-credits", _("translator-credits"),
                          "license-type", GTK_LICENSE_GPL_3_0,
-                         "logo-icon-name", "sysprof",
+                         "logo-icon-name", "org.gnome.Sysprof",
                          "program-name", _("Sysprof"),
                          "version", PACKAGE_VERSION,
                          "website", "https://wiki.gnome.org/Apps/Sysprof",
